package com.steerinfo.ems.njformula.controller;

import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import com.steerinfo.ems.njformula.model.NjFormula;
import com.steerinfo.ems.njformula.service.INjFormulaService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.math.BigDecimal;

/**
 * NjFormula RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-10-14 02:53
 * 类描述
 * 修订历史：
 * 日期：2021-10-14
 * 作者：generator
 * 参考：
 * 描述：NjFormula RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/njformulas")
public class NjFormulaController extends BaseRESTfulController {

    @Autowired
    INjFormulaService njFormulaService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("njformula:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<NjFormula> list = njFormulaService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("njformula:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<NjFormula> list = njFormulaService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据NjFormula对象创建")
    @ApiImplicitParam(name = "njFormula", value = "详细实体njFormula", required = true, dataType = "NjFormula")
    //@RequiresPermissions("njformula:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute NjFormula model){
        NjFormula njFormula = njFormulaService.add(model);
        return success(njFormula);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("njformula:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable String id){
        NjFormula njFormula = njFormulaService.getById(id);
        return success(njFormula);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的njFormula信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String"),
        @ApiImplicitParam(name = "njFormula", value = "详细实体njFormula", required = true, dataType = "NjFormula")
    })
    //@RequiresPermissions("njformula:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable String id, @RequestBody NjFormula model){
        model.setId(id);
        NjFormula njFormula = njFormulaService.modify(model);
        return success(njFormula);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "String")
    //@RequiresPermissions("njformula:delete")
    @DeleteMapping(value = "/{id}")//String
    public RESTfulResult delete(@PathVariable String id){
    	List<String> list = Arrays.asList(id.split(","));
    	if(ListUtils.isNotEmpty(list)) {
	    	List<String> ids = ListUtils.convertList(list);
			  njFormulaService.delete(ids);
    	}
      return success();
    }
}
