package com.steerinfo.ems.rttable.controller;

import com.steerinfo.ems.rttable.model.Rttable;
import com.steerinfo.ems.rttable.service.IRttableService;
import com.steerinfo.framework.controller.BaseRESTfulController;
import com.steerinfo.framework.controller.RESTfulResult;
import com.steerinfo.framework.service.pagehelper.PageList;
import com.steerinfo.framework.utils.collection.ListUtils;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * Rttable RESTful接口：
 * @author generator
 * @version 1.0-SNAPSHORT 2021-10-15 03:37
 * 类描述
 * 修订历史：
 * 日期：2021-10-15
 * 作者：generator
 * 参考：
 * 描述：Rttable RESTful接口
 * @see null
 * @Copyright 湖南视拓信息技术股份有限公司. All rights reserved.
 */
@RestController
@RequestMapping("/${api.version}/rttables")
public class RttableController extends BaseRESTfulController {

    @Autowired
    IRttableService rttableService;

    @ApiOperation(value="获取列表", notes="分页查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("rttable:view")
    @GetMapping(value = "/")
    public RESTfulResult list(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<Rttable> list = rttableService.queryForPage(parmas, pageNum, pageSize);
        return success(list);
    }

    @ApiOperation(value="获取列表", notes="分页模糊查询")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "pageNum", value = "查询页数", required = false, dataType = "Integer"),
        @ApiImplicitParam(name = "pageSize", value = "每页记录数", required = false, dataType = "Integer")
    })
    //@RequiresPermissions("rttable:view")
    @GetMapping(value = "/like/")
    public RESTfulResult listLike(@RequestParam HashMap parmas,Integer pageNum, Integer pageSize){
        PageList<Rttable> list = rttableService.queryLikeForPage(parmas, pageNum, pageSize);
        return success(list);
    }
    
    @ApiOperation(value="创建", notes="根据Rttable对象创建")
    @ApiImplicitParam(name = "rttable", value = "详细实体rttable", required = true, dataType = "Rttable")
    //@RequiresPermissions("rttable:create")
    @PostMapping(value = "/")
    public RESTfulResult add(@ModelAttribute Rttable model){
        Rttable rttable = rttableService.add(model);
        return success(rttable);
    }

    @ApiOperation(value="获取详细信息", notes="根据url的id来获取详细信息")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "BigDecimal")
    //@RequiresPermissions("rttable:view")
    @GetMapping(value = "/{id}")
    public RESTfulResult get(@PathVariable BigDecimal id){
        Rttable rttable = rttableService.getById(id);
        return success(rttable);
    }

    @ApiOperation(value="更新详细信息", notes="根据url的id来指定更新对象，并根据传过来的rttable信息来更新详细信息")
    @ApiImplicitParams({
        @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "BigDecimal"),
        @ApiImplicitParam(name = "rttable", value = "详细实体rttable", required = true, dataType = "Rttable")
    })
    //@RequiresPermissions("rttable:update")
    @PutMapping(value = "/{id}", produces  = "application/json;charset=UTF-8")
    public RESTfulResult update(@PathVariable BigDecimal id, @RequestBody Rttable model){
        model.setId(id);
        Rttable rttable = rttableService.modify(model);
        return success(rttable);
    }

    @ApiOperation(value="删除", notes="根据url的id来指定删除对象")
    @ApiImplicitParam(paramType = "path", name = "id", value = "ID", required = true, dataType = "BigDecimal")
    //@RequiresPermissions("rttable:delete")
    @DeleteMapping(value = "/{id}")//BigDecimal
    public RESTfulResult delete(@PathVariable String id){
    	List<String> list = Arrays.asList(id.split(","));
    	if(ListUtils.isNotEmpty(list)) {
	    	List<BigDecimal> ids = ListUtils.convertList(list);
			  rttableService.delete(ids);
    	}
      return success();
    }

}
